package fr.inra.moulon.svgutils.svggelimage;

import java.net.URL;

import org.w3c.dom.Element;

import fr.inra.moulon.svgutils.SvgGraphics;

public class SvgSpot {

	protected SvgSpotSymbol symbol;

	protected float x, y;

	protected String label;
	protected String label_css_class;

	protected SvgGelImage gel_image;

	protected boolean is_highlighted;

	private String spotIdentifier;

	private String group;

	public SvgSpot(SvgGelImage gel_image, float x, float y,
			SvgSpotSymbol currentSymbol) {
		this.gel_image = gel_image;
		this.symbol = currentSymbol;
		this.x = x;
		this.y = y;
		this.label_css_class = "slabelf";
		is_highlighted = false;
		spotIdentifier = null;
		group = gel_image.getCurrentSpotGroup();
	}

	public float getX() {
		return this.x;
	}

	public float getY() {
		return this.y;
	}

	public Element drawSvgSpot() throws Exception {
		/*
		 * <use x="45" y="10" width="10" height="10" xlink:href="#MySymbol" />
		 */
		gel_image.getSvgGraphic().goToGroup(group);
		Element spot = gel_image.getSvgGraphic().use(this.x - 50, this.y - 50,
				this.symbol.getElement());

		if ((this.getId() != null) && (gel_image.getSpotBaseUrl() != null)) {
			String spotUrl = gel_image.getSpotBaseUrl().toString()
					+ this.getId();
			Element a = gel_image.getSvgGraphic().anchor(new URL(spotUrl));
			// onclick="window.open(this.href); return false;"
			a.setAttributeNS(
					null,
					"onclick",
					"window.open('"
							+ a.getAttributeNS(SvgGraphics.xlinkNS, "href")
							+ "'); return false;");
			a.appendChild(spot);
			spot = a;
		}

		if (this.label != null) {
			// gel_image.getSvgGraphic().goToGroup("slabel");

			Element label = gel_image.getSvgGraphic().text(this.x,
					this.y - (this.gel_image.get_spot_label_size() / 2),
					this.label, null);

		}

		return spot;

	}

	public void setId(String spotIdentifier) {
		this.spotIdentifier = spotIdentifier;
	}

	private String getId() {
		return spotIdentifier;
	}

	public boolean isHighLighted() {
		return (this.is_highlighted);
	}

	public void setHighLight(boolean to_highlight) {
		this.is_highlighted = to_highlight;
	}

	public void setSymbol(SvgSpotSymbol symbol) {
		this.symbol = symbol;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public String getLabel() {
		return (this.label);
	}
}
