package fr.inra.moulon.svgutils.svgspectrum;

public class SvgIon {

	SvgIonSpecies ionSpecies;
	protected Integer number;
	protected float mz;
	protected Float assignedPeakIntensityRatio;

	public Float getAssignedPeakIntensityRatio() {
		return assignedPeakIntensityRatio;
	}

	public void setAssignedPeakIntensityRatio(Float assignedPeakIntensityRatio) {
		this.assignedPeakIntensityRatio = assignedPeakIntensityRatio;
	}

	static float mhplus = (float) 1.00794;
	static float moxygen = (float) 15.9951;
	// H2O loss:
	static float mh2o = (mhplus * 2) + moxygen;
	// amine loss:
	static float mnh3 = (float) ((mhplus * 3) + 14.00674);
	static float mco = (float) 27.99491;

	public SvgIon() {
		number = null;
		mz = 0;
		assignedPeakIntensityRatio = null;
		ionSpecies = null;
	}

	public void setIonY1() {
		ionSpecies = SvgIonSpecies.getY1Instance();
		this.mz = SvgIon.mh2o + SvgIon.mhplus;
	}

	public void setIonB1() {
		ionSpecies = SvgIonSpecies.getB1Instance();
		this.mz = SvgIon.mhplus;
	}

	public void setIonY2() {
		ionSpecies = SvgIonSpecies.getY2Instance();
		this.mz = mhplus;
		this.mz += SvgIon.mh2o + SvgIon.mhplus;
	}

	public void setIonB2() {
		ionSpecies = SvgIonSpecies.getB2Instance();

		this.mz = mhplus;
		this.mz += SvgIon.mhplus;
	}

	public void setIonYstar1() {
		ionSpecies = SvgIonSpecies.getYstar1Instance();
		this.mz = -mnh3;
		this.mz += SvgIon.mh2o + SvgIon.mhplus;
	}

	public void setIonYstar2() {
		ionSpecies = SvgIonSpecies.getYstar2Instance();
		this.mz = -mnh3 + mhplus;
		this.mz += SvgIon.mh2o + SvgIon.mhplus;
	}

	public void setIonYO1() {
		ionSpecies = SvgIonSpecies.getYO1Instance();
		this.mz = -mh2o;
		this.mz += SvgIon.mh2o + SvgIon.mhplus;
	}

	public void setIonYO2() {
		ionSpecies = SvgIonSpecies.getYO2Instance();
		this.mz = -mh2o + mhplus;
		this.mz += SvgIon.mh2o + SvgIon.mhplus;
	}

	public void setIonBstar1() {
		ionSpecies = SvgIonSpecies.getBstar1Instance();
		this.mz = -mnh3;
		this.mz += SvgIon.mhplus;
	}

	public void setIonBstar2() {
		ionSpecies = SvgIonSpecies.getBstar2Instance();
		this.mz = -mnh3 + mhplus;
	}

	public void setIonBO1() {
		ionSpecies = SvgIonSpecies.getBO1Instance();
		this.mz = -mh2o;
		this.mz += SvgIon.mhplus;
	}

	public void setIonBO2() {
		ionSpecies = SvgIonSpecies.getBO2Instance();
		this.mz = -mh2o + mhplus;
		this.mz += SvgIon.mhplus;
	}

	public void setIonA1() {
		ionSpecies = SvgIonSpecies.getA1Instance();

		this.mz = -mco;
		this.mz += SvgIon.mhplus;
	}

	public void setIonC1() {
		ionSpecies = SvgIonSpecies.getC1Instance();
	}

	public void setIonC2() {
		ionSpecies = SvgIonSpecies.getC2Instance();
	}

	public void setIonZ1() {
		ionSpecies = SvgIonSpecies.getZ1Instance();
	}

	public void setIonZ2() {
		ionSpecies = SvgIonSpecies.getZ2Instance();
	}

	public int getNumber() {
		return number;

	}

	public void setNumber(int number) {
		this.number = number;
	}

	public float getMz() {
		return mz;
	}

	public void setMz(float mz) {
		this.mz = mz;
	}

	public String getName() {
		String name = this.ionSpecies.getName();
		if (number != null) {
			name += number;
		}
		return name;
	}

	public void setSequenceMass(float mass) {
		this.mz += mass;

		this.mz = this.mz / this.ionSpecies.getZ();
	}

	public SvgIonSpecies getSvgIonSpecies() {
		return this.ionSpecies;
	}


	@Override
	public boolean equals(Object aThat) {
		// System.out.println("coucou");
		if (this == aThat)
			return true;
		if (!(aThat instanceof SvgIon))
			return false;
		SvgIon ion = (SvgIon) aThat;
		if (number.equals(ion.number)) {
			if (this.ionSpecies.equals(ion.ionSpecies)) {
				return true;
			}
		}
		return false;
	}

}
