package fr.inra.moulon.svgutils.svgspectrum;

import java.util.HashMap;

import org.w3c.dom.Element;
import org.w3c.dom.Text;

import fr.inra.moulon.xhtmlutils.XhtmlDocument;
import fr.inra.moulon.xhtmlutils.XhtmlElementBase;

public class SvgIonSpecies implements Comparable<SvgIonSpecies> {
	private static HashMap<String, SvgIonSpecies> hashIonSpecies = new HashMap<String, SvgIonSpecies>(
			0);
	protected String type;
	protected Integer z;
	protected int priority;
	protected boolean isCside; // for a and b : false for y : true

	private SvgIonSpecies() {
		type = null;
		z = null;
	}

	public String getCssClassname() {
		String classname = type + z;
		classname = classname.replace("*", "star");
		return classname;
	}

	public String getTypeZ() {
		return type + z;
	}

	public String getType() {
		return type;
	}

	protected void setType(String type) {
		this.type = type;
	}

	public int getZ() {
		return z;
	}

	protected void setZ(int iz) {
		this.z = iz;
	}

	public String getName() {
		String name = type;
		if (z > 1) {
			for (int i = 0; i < z; i++) {
				name += '+';
			}
		}
		return name;
	}

	/**
	 * tell if the current ion has priority over an other
	 * 
	 */
	public boolean priority_over(SvgIonSpecies toCompare) {

		// y >> b >> y++ >> b++ >> a >> bO >> bO++
		if (this.priority > toCompare.getPriority()) {
			return (false);
		}
		return (true);
	}

	public int getPriority() {
		return priority;
	}

	public static SvgIonSpecies getY1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "y1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("y");
			ionSpecies.isCside = true;
			ionSpecies.setZ(1);
			ionSpecies.priority = 1;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getB1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "b1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("b");
			ionSpecies.isCside = false;
			ionSpecies.setZ(1);
			ionSpecies.priority = 2;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;

	}

	public static SvgIonSpecies getY2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "y2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("y");
			ionSpecies.isCside = true;
			ionSpecies.setZ(2);
			ionSpecies.priority = 3;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public boolean isCside() {
		return isCside;
	}

	public static SvgIonSpecies getB2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "b2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("b");
			ionSpecies.isCside = false;
			ionSpecies.setZ(2);
			ionSpecies.priority = 4;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getYstar1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "y*1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("y*");
			ionSpecies.isCside = true;
			ionSpecies.setZ(1);
			ionSpecies.priority = 5;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getYstar2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "y*2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("y*");
			ionSpecies.isCside = true;
			ionSpecies.setZ(2);
			ionSpecies.priority = 6;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getYO1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "yO1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("yO");
			ionSpecies.isCside = true;
			ionSpecies.setZ(1);
			ionSpecies.priority = 7;
			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getYO2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "yO2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("yO");
			ionSpecies.isCside = true;
			ionSpecies.setZ(2);
			ionSpecies.priority = 8;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getBstar1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "b*1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("b*");
			ionSpecies.isCside = false;
			ionSpecies.setZ(1);
			ionSpecies.priority = 9;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getBstar2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "b*2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("b*");
			ionSpecies.isCside = false;
			ionSpecies.setZ(2);
			ionSpecies.priority = 10;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getBO1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "bO1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("bO");
			ionSpecies.isCside = false;
			ionSpecies.setZ(1);
			ionSpecies.priority = 11;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getBO2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "bO2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("bO");
			ionSpecies.isCside = false;
			ionSpecies.setZ(2);
			ionSpecies.priority = 12;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getA1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "a1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("a");
			ionSpecies.isCside = false;
			ionSpecies.setZ(1);
			ionSpecies.priority = 13;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getC1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "c1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("c");
			ionSpecies.isCside = false;
			ionSpecies.setZ(1);
			ionSpecies.priority = 13;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getC2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "c2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("c");
			ionSpecies.isCside = false;
			ionSpecies.setZ(2);
			ionSpecies.priority = 13;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getZ1Instance() {
		SvgIonSpecies ionSpecies;
		String key = "z1";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("z");
			ionSpecies.isCside = true;
			ionSpecies.setZ(1);
			ionSpecies.priority = 13;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public static SvgIonSpecies getZ2Instance() {
		SvgIonSpecies ionSpecies;
		String key = "z2";
		if (hashIonSpecies.containsKey(key)) {
			ionSpecies = hashIonSpecies.get(key);
		} else {
			ionSpecies = new SvgIonSpecies();
			ionSpecies.setType("z");
			ionSpecies.isCside = true;
			ionSpecies.setZ(2);
			ionSpecies.priority = 13;

			hashIonSpecies.put(key, ionSpecies);
		}

		return ionSpecies;
	}

	public int compareTo(SvgIonSpecies toCompare) {
		if (this.priority > toCompare.getPriority()) {
			return (1);
		}
		if (this.priority < toCompare.getPriority()) {
			// -1 less than
			return (-1);
		}
		return 0; // equal
	}

	@Override
	public int hashCode() {
		String strCode = "" + type + "-" + z;
		return strCode.hashCode();
	}

	@Override
	public boolean equals(Object aThat) {
		// System.out.println("coucou");
		if (this == aThat)
			return true;
		if (!(aThat instanceof SvgIonSpecies))
			return false;
		if (this.hashCode() == aThat.hashCode()) {
			return true;
		}
		return false;
	}

	public String getCssColor() {
		String color = "black";
		if (type.equals("a")) {
			color = "green";
		} else if ((type.equals("bO") || (type.equals("bstar")))) {
			color = "#ff00ff";
		} else if ((type.equals("yO") || (type.equals("ystar")))) {
			color = "orange";
		} else if (type.equals("b")) {
			color = "blue";
		} else if (type.equals("y")) {
			color = "red";
		}
		return color;
	}

	public void printTo(XhtmlElementBase element) {
		Text node = element.getXhtmlDocument().getDocument()
				.createTextNode(this.type);
		element.appendChild(node);
		if (this.z > 1) {
			Element exp = element.getXhtmlDocument().getDocument()
					.createElementNS(XhtmlDocument.xhtmlNS, "sup");
			element.appendChild(exp);
			String strExp = "";
			for (int i = 0; i < this.z; i++) {
				strExp += "+";
			}
			exp.setTextContent(strExp);
		}
		//element.appendChild(node);
	}
}
