package fr.inra.moulon.svgutils.svgspectrum;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import fr.inra.moulon.svgutils.SvgGraphics;
import fr.inra.moulon.svgutils.SvgGroup;

import org.w3c.dom.Element;

public class SvgPeptideSequence implements Iterable<SvgAminoAcid> {

	private Vector<SvgAminoAcid> sequence;
	private int exp_z;
	private int fontSize;


	public int getFontSize() {
		return fontSize;
	}

	public void setFontSize(int fontSize) {
		this.fontSize = fontSize;
	}

	public SvgPeptideSequence(String str_sequence) throws Exception {
		super();
		this.sequence = new Vector<SvgAminoAcid>();
		exp_z = 2;
		fontSize = 20;
		// System.out.print("SvgPeptideSequence(String str_sequence) coucou " +
		// str_sequence);
		for (Character aacar : str_sequence.toCharArray()) {
			// System.out.print("SvgPeptideSequence(String str_sequence)" +
			// aacar+ " " + str_sequence);
			SvgAminoAcid aa = new SvgAminoAcid(aacar);
			this.sequence.add(aa);
		}
	}

	public String toString() {
		String tmpstring = new String();
		for (SvgAminoAcid aa : this.sequence) {
			// System.out.print("SvgPeptideSequence(String str_sequence)" +
			// aacar+ " " + str_sequence);
			tmpstring += aa.toString();
		}
		return tmpstring;
	}

	public int getExp_z() {
		return exp_z;
	}

	public void setExp_z(int expZ) {
		exp_z = expZ;
	}

	public int size() {
		return sequence.size();
	}

	public SvgAminoAcid get(int i) {
		return (this.sequence.get(i));
	}

	public SvgAminoAcid getAA(int i) {
		return (this.sequence.get(i - 1));
	}

	public float getSubCterMass(int length) throws Exception {
		if (length > sequence.size()) {
			throw new Exception("length too long in getSubCterMass :" + length
					+ " for maximum length : " + sequence.size());
		}
		float mass = 0;
		for (int i = (sequence.size() - 1); i >= (length - 1); i--) {
			mass += sequence.get(i).getMass();
		}
		return mass;
	}

	public float getSubNterMass(int length) throws Exception {
		if (length > sequence.size()) {
			throw new Exception("length too long in getSubCterMass :" + length
					+ " for maximum length : " + sequence.size());
		}
		float mass = 0;
		for (int i = 0; i < length; i++) {
			mass += sequence.get(i).getMass();
		}
		return mass;
	}

	public float getMass() {
		float mass = 0;
		for (SvgAminoAcid aa : sequence) {
			mass += aa.getMass();
		}
		return mass;
	}

	public Vector<SvgIon> getIonTable() {
		Vector<SvgIon> iontable = new Vector<SvgIon>();
		/*
		 * 
		 * public void setIonC1() { setType("c"); setZ(1); this.priority = 13; }
		 * 
		 * public void setIonC2() { setType("c"); setZ(2); this.priority = 13; }
		 * 
		 * public void setIonZ1() { setType("z"); setZ(1); this.priority = 13; }
		 * 
		 * public void setIonZ2() { setType("z"); setZ(2); this.priority = 13; }
		 */
		// Nter ions
		float mass = 0;
		int num = 1;
		SvgIon tmp_ion;
		for (int i = 0; i < sequence.size() - 1; i++, num++) {
			mass += sequence.get(i).getMass();

			// ion b, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonB1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion a, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonA1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion b, z = 2
			if (this.exp_z > 1) {
				tmp_ion = new SvgIon();
				tmp_ion.setIonB2();
				tmp_ion.setNumber(num);
				tmp_ion.setSequenceMass(mass);

				iontable.add(tmp_ion);
			}

			// ion b*, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonBstar1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion b*, z = 2
			if (this.exp_z > 1) {
				tmp_ion = new SvgIon();
				tmp_ion.setIonBstar2();
				tmp_ion.setNumber(num);
				tmp_ion.setSequenceMass(mass);

				iontable.add(tmp_ion);
			}

			// ion bO, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonBO1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion bO, z = 2
			if (exp_z > 1) {
				tmp_ion = new SvgIon();
				tmp_ion.setIonBO2();
				tmp_ion.setNumber(num);
				tmp_ion.setSequenceMass(mass);

				iontable.add(tmp_ion);
			}

		}
		// Cter ions
		mass = 0;
		num = 1;
		for (int i = (sequence.size() - 1); i > 0; i--, num++) {
			mass += sequence.get(i).getMass();
			// ion y, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonY1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion y, z = 2
			if (exp_z > 1) {
				tmp_ion = new SvgIon();
				tmp_ion.setIonY2();
				tmp_ion.setNumber(num);
				tmp_ion.setSequenceMass(mass);

				iontable.add(tmp_ion);
			}

			// ion y*, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonYstar1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion y*, z = 2
			if (exp_z > 1) {
				tmp_ion = new SvgIon();
				tmp_ion.setIonYstar2();
				tmp_ion.setNumber(num);
				tmp_ion.setSequenceMass(mass);

				iontable.add(tmp_ion);
			}

			// ion yO, z = 1
			tmp_ion = new SvgIon();
			tmp_ion.setIonYO1();
			tmp_ion.setNumber(num);
			tmp_ion.setSequenceMass(mass);

			iontable.add(tmp_ion);

			// ion yO, z = 2
			if (exp_z > 1) {
				tmp_ion = new SvgIon();
				tmp_ion.setIonYO2();
				tmp_ion.setNumber(num);
				tmp_ion.setSequenceMass(mass);

				iontable.add(tmp_ion);
			}
		}

		return iontable;
	}

	public float svgWidth() {
		return (this.fontSize * (this.sequence.size()));
	}

	public Element drawSvgPeptideSequence(SvgGraphics svg_graphic,
			float xcoord, float ycoord) throws Exception {
		svg_graphic.group("svgsequence", "");
		SvgGroup group_sequence = svg_graphic.goToGroup("svgsequence");

		// Element text = SvgGraphics.svgText(svg_graphic.getDocument(),
		// group_sequence.getElement(), xcoord, ycoord, "");
		// xcoord += this.fontSize;
		for (SvgAminoAcid aa : this.sequence) {
			// SvgGraphics.svgTspan(svg_graphic.getDocument(), text, xcoord,
			// ycoord, aa.toString(), "");
			aa.svgAssignedIon(svg_graphic, xcoord, ycoord, this.fontSize);
			xcoord += this.fontSize;

		}

		svg_graphic.setCssClass("#svgsequence tspan",
				"font-size:" + this.getFontSize()
						+ "px;stroke:#000000;fill:#000000;text-anchor:middle;");

		return group_sequence.getElement();

	}

	public void addAssignedIon(SvgIon ion) {
		if (ion.getSvgIonSpecies().isCside() == false) {
			this.sequence.get(ion.getNumber() - 1).addAssignedIon(ion);
		} else {
			this.sequence.get(sequence.size() - ion.getNumber())
					.addAssignedIon(ion);
		}
	}

	public ArrayList<SvgIon> getAssignedIons() {
		ArrayList<SvgIon> iontable = new ArrayList<SvgIon>();
		for (int i = 0; i < sequence.size(); i++) {
			for (SvgIon ion : sequence.get(i).getAssignedIons()) {
				iontable.add(ion);

			}
		}
		return (iontable);
	}

	public TreeSet<SvgIonSpecies> getNterSvgIonSpeciesList() {
		TreeSet<SvgIonSpecies> speciesList = new TreeSet<SvgIonSpecies>();

		Vector<SvgIon> ionTable = getIonTable();
		for (SvgIon ion : ionTable) {
			SvgIonSpecies ionSpecies = ion.getSvgIonSpecies();
			if (ionSpecies.isCside() == false) {
				speciesList.add(ionSpecies);
			}
		}
		return speciesList;

	}
	public TreeSet<SvgIonSpecies> getCterSvgIonSpeciesList() {
		TreeSet<SvgIonSpecies> speciesList = new TreeSet<SvgIonSpecies>();

		Vector<SvgIon> ionTable = getIonTable();
		for (SvgIon ion : ionTable) {
			SvgIonSpecies ionSpecies = ion.getSvgIonSpecies();
			if (ionSpecies.isCside() == true) {
				speciesList.add(ionSpecies);
			}
		}
		return speciesList;

	}

	public Iterator<SvgAminoAcid> iterator() {
		return this.sequence.iterator();
	}

	public HashSet<SvgAaMod> getSvgAaModSet() {
		
		HashSet<SvgAaMod> svgAaModList=new HashSet<SvgAaMod>(0);
		for (SvgAminoAcid aa : this.sequence) {
			for (SvgAaMod mod: aa.getSvgAaModList()) {
				svgAaModList.add(mod);
			}
		}
		return (svgAaModList);
	}

}
