package fr.inra.pappso.xtandempipeline;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.grouping.HashPeptide;
import fr.inra.pappso.xtandempipeline.grouping.HashPeptideSet;
import fr.inra.pappso.xtandempipeline.grouping.HashProtein;
import fr.inra.pappso.xtandempipeline.grouping.HashProteinSet;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSet;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;

public class XtandemPipelineSession {
	
	private static XtandemPipelineSession session=null;
	
	private static final Logger logger = Logger.getLogger(XtandemPipelineSession.class);
	
	private HashSampleScanSet sampleScanSet = new HashSampleScanSet();
	private HashPeptideSet peptideSet = new HashPeptideSet();
	private HashProteinSet proteinSet = new HashProteinSet();
	
	private ArrayList<Identification> identificationList = new ArrayList<Identification>();
	
	private config conf;
	
	private DataType_Base dataType;

	private XtandemPipelineSession(){
		conf = new config();
		dataType = new DataType_Base(conf);
	}
	
	static public XtandemPipelineSession getInstance(){
		if(session==null){
			session = new XtandemPipelineSession();
		}
		return(session);
	}
	
	public HashSampleScan getHashSampleScanInstance(int scan, MsRun msRun) {
		return sampleScanSet.getInstance(scan, msRun);
	}

	public HashPeptide getHashPeptideInstance(HashSampleScan hashScan,
			String get_LI_sequence) throws Exception {
		return peptideSet.getInstance(hashScan, get_LI_sequence);
	}

	public HashProtein getHashProteinInstance(String accession) {
		return proteinSet.getInstance(accession);
	}
	
	public ArrayList<Identification> getIdentificationList() {
		return identificationList;
	}
	
	public void setIdentificationList(ArrayList<Identification> identifis) {
		this.identificationList = identifis;
	}

	public config getConfig() {
		return this.conf;
	}
	public DataType_Base getDataTypeBase() {
		return this.dataType;
	}

	public void setDataTypeBase(DataType_Base type) {
		this.dataType = type;
	}

	public void clearIdentificationResult() {
		logger.info("Clearing session");
		sampleScanSet.clear();
		identificationList.clear();
		peptideSet.clear();
		proteinSet.clear();
		identificationList = new ArrayList<Identification>();
		sampleScanSet=new HashSampleScanSet();
		peptideSet=new HashPeptideSet();
		proteinSet=new HashProteinSet();
		
		this.dataType=new DataType_Base(conf);
		
		System.gc();
		
	}
	
}
