package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;
import java.util.Vector;


import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.filter_print.filter_index_and_order_match_scan;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_recalcul_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_peptide_evalue;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem;


public class combined extends process {

	private Vector<File> xtandem;

//	private ArrayList<Identification> identifications = XtandemPipelineSession
//	.getInstance().getIdentificationList();

	public combined(config conf) {
		super(conf);
	}

	protected void processing() throws Exception {
		logger.info("Process result in Combine mode");
		
		// Ecrire le process et penser à relever des StopException
		xtandem handler = new xtandem();
		for (int i = 0; i < xtandem.size(); i++) {
			// parsing
			this.set_view("Parsing file : " + (i + 1) + "/" + xtandem.size()
					+ "\n" + xtandem.get(i).getName());
			handler.set_sample(xtandem.get(i));
			new parse_file(handler, xtandem.get(i));

			// verification pas arreté
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();
		}

		// filtering
		this.set_view("Parsing finished\n" + "Filtering peptides");
		Identification ident = handler.get_identifications();
		
		//on mets à jour les liens
		ident.filtered_match(new filter_index_and_order_match_scan());

		//filtre evalue peptide
		ident.filtered_match(new filter_peptide_evalue((conf
				.get_peptide_evalue())));
		
		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();
		
		// calcul de la evalue globale et modification evalue sample
		ident.filtered_match(new filter_recalcul_protein_evalue());
		
		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Parsing finished\n" + "Filtering proteins");
		if (conf.get_protein_filter_to_complete()) {
			ident.filtered_match(new filter_protein_evalue(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number(conf
					.get_peptide_number()));
		} else {
			ident.filtered_match(new filter_protein_evalue_to(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number_to(conf
					.get_peptide_number()));
		}

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// creation des submatchs
		this.set_view("Grouping\n"
				+ "Be patient, this step can take some time");
		ident.performedGroupSet();
		
		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// elimination des protéines contaminantes
		ident.remove_contaminant_submatch(conf.get_database_filter());

		//ArrayList<Identification> identifications = XtandemPipelineSession.getInstance().getIdentificationList();
		XtandemPipelineSession.getInstance().getIdentificationList().add(ident);

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

	}

	public void setXtandem(Vector<File> xtandem) {
		this.xtandem = xtandem;
		this.set_max_progress(xtandem.size()+5);
	}

	public Vector<File> getXtandem() {
		return xtandem;
	}

}
