package fr.inra.moulon.svgutils;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.w3c.dom.*;

import javax.imageio.ImageIO;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

//This is a small proof-of-concept graphics class that
// provides method calls for the creation of the following
// DOM tree nodes:
//  A simple element with no attributes.
//  A linear gradient element.
//  An ellipse
//  A circle

//Each method receives a reference to the overall document
// along with a reference to the parent for the new node.
//When the method returns, the new node has been appended
// to the parent node.

public class SvgGraphics implements SvgCommon {

	public static String svgNS = "http://www.w3.org/2000/svg";
	public static String xlinkNS = "http://www.w3.org/1999/xlink";
	public static String xmlNS = "http://www.w3.org/2000/xmlns/";

	private Document svgdoc;
	private Element svgroot;
	private Element currentNode;
	private Element style;
	private CDATASection cdataStyle;
	private Element defs;

	private Float width;
	private Float height;

	HashMap<String, SvgGroup> groups = new HashMap<String, SvgGroup>();
	HashMap<String, String> cssClasses = new HashMap<String, String>();
	HashMap<String, Element> hash_defs = new HashMap<String, Element>();

	public SvgGraphics(Float width, Float height) {
		this.width = width;
		this.height = height;

		// String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
		// DOMImplementation impl = SVGDOMImplementation.getDOMImplementation();
		// svgdoc = (Document) impl.createDocument(svgNS, "svg", null);

		svgdoc = createDocument();
		svgroot = SvgGraphics.getRootNode(svgdoc, width, height);
		currentNode = svgroot;
		// <style type="text/css"> ... </style>
		style = svgStyle(svgdoc, svgroot);
		defs = svgDefs(svgdoc, svgroot);
		cdataStyle = this.svgdoc.createCDATASection(" ");
		style.appendChild(cdataStyle);
		this.setCssClass(
				"svg",
				"fill:none;fill-rule:evenodd;stroke:#000000;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1");
	}

	public String getSvgNS() {
		return SvgGraphics.svgNS;
	}

	public Element getSvgRoot() {
		return svgroot;
	}

	public static Element svgCircle(Document document, Element parent,
			float cx, float cy, float r, String classname) {
		// <circle cx="100" cy="50" r="40"
		Element circle = document.createElementNS(svgNS, "circle");
		circle.setAttributeNS(null, "cx", "" + cx);
		circle.setAttributeNS(null, "cy", "" + cy);
		circle.setAttributeNS(null, "r", "" + r);
		if ((classname != null) && (!classname.equals(""))) {
			circle.setAttributeNS(null, "class", classname);
		}
		parent.appendChild(circle);
		return circle;

	}

	public static Element svgDefs(Document document, Element parent) {

		// create the rectangle
		Element rect = document.createElementNS(svgNS, "defs");
		parent.appendChild(rect);
		return rect;
	}

	public static Element svgStyle(Document document, Element parent) {

		// create the rectangle
		Element rect = document.createElementNS(svgNS, "style");
		parent.appendChild(rect);
		rect.setAttributeNS(null, "type", "text/css");
		return rect;
	}

	public static Element svgRect(Document document, Element parent,
			float xcoord, float ycoord, float width, float height,
			String classname) {

		// create the rectangle
		Element rect = document.createElementNS(svgNS, "rect");
		parent.appendChild(rect);
		rect.setAttributeNS(null, "x", "" + xcoord);
		rect.setAttributeNS(null, "y", "" + ycoord);
		rect.setAttributeNS(null, "width", "" + width);
		rect.setAttributeNS(null, "height", "" + height);
		if (!classname.equals("")) {
			rect.setAttributeNS(null, "class", classname);
		}

		return rect;
	}

	public static Element svgTspan(Document document, Element parent,
			Float xCoor, Float yCoor, String content, String classname) {

		// create the rectangle
		Element rect = document.createElementNS(svgNS, "tspan");
		parent.appendChild(rect);
		if (xCoor != null)
			rect.setAttributeNS(null, "x", "" + xCoor);
		if (yCoor != null)
			rect.setAttributeNS(null, "y", "" + yCoor);
		if (!classname.equals("")) {
			rect.setAttributeNS(null, "class", classname);
		}
		rect.appendChild(document.createTextNode(content));
		return rect;
	}

	public static Element svgText(Document document, Element parent,
			float xCoor, float yCoor, String classname) {

		// create the rectangle
		Element rect = document.createElementNS(svgNS, "text");
		parent.appendChild(rect);
		rect.setAttributeNS(null, "x", "" + xCoor);
		rect.setAttributeNS(null, "y", "" + yCoor);
		if ((classname != null) && (!classname.equals(""))) {
			rect.setAttributeNS(null, "class", classname);
		}
		// rect.appendChild(document.createTextNode(content));
		return rect;
	}

	public static Element svgSpan(Document document, Element parent, int xCoor,
			int yCoor, String classname) {

		// create the rectangle
		Element rect = document.createElementNS(svgNS, "span");
		parent.appendChild(rect);
		rect.setAttributeNS(null, "x", "" + xCoor);
		rect.setAttributeNS(null, "y", "" + yCoor);
		if (!classname.equals("")) {
			rect.setAttributeNS(null, "class", classname);
		}

		return rect;
	}

	public static Element svgLine(Document document, Element parent,
			float xBeginCoor, float yBeginCoor, float width, float height,
			String classname) {

		// create the rectangle
		Element line = document.createElementNS(svgNS, "path");
		parent.appendChild(line);
		line.setAttributeNS(null, "d", "M " + xBeginCoor + "," + yBeginCoor
				+ " L " + width + "," + height);
		if (classname != null) {
			if (!classname.equals("")) {
				line.setAttributeNS(null, "class", classname);
			}
		}

		return line;
	}

	public static Element svgAnchor(Document document, Element parent, URL href) {
		// <a xlink:href="/" xlink:title="Go Home">
		Element anchor = document.createElementNS(svgNS, "a");
		parent.appendChild(anchor);
		anchor.setAttributeNS(xlinkNS, "xlink:href", href.toString());
		return anchor;
	}

	public static Element svgGroup(Document document, Element parent,
			String id, String classname) {

		// create the rectangle
		Element group = document.createElementNS(svgNS, "g");
		parent.appendChild(group);
		group.setAttributeNS(null, "id", "" + id);
		if (classname != null) {
			if (!classname.equals("")) {
				group.setAttributeNS(null, "class", classname);
			}
		}

		return group;
	}

	// ----------------------------------------------------//

	// This method returns a reference to a root node that
	// has already been appended to the document.
	public static Element getRootNode(Document document, Float width,
			Float height) {
		// Create the root node named svg and append it to
		// the document.
		/*
		 * Element svg = (Element)document.createElement("svg");
		 * document.appendChild(svg);
		 */

		// get the root element (the svg element)
		Element svg = document.getDocumentElement();

		// set the width and height attribute on the svg root element
		if (width != null) {
			svg.setAttributeNS(null, "width", "" + width);
		}
		if (height != null) {
			svg.setAttributeNS(null, "height", "" + height);
		}

		/*
		 * //Set some attributes on the root node that are // required for
		 * proper rendering. Note that the // approach used here is somewhat
		 * different from the // approach used in the earlier program named
		 * Svg01, // particularly with regard to the style. svg.setAttribute(
		 * "xmlns","http://www.w3.org/2000/svg");
		 * svg.setAttribute("version","1.1"); svg.setAttribute("style","width:"
		 * + width + ";height:" + height + ";position:" + position + ";top:" +
		 * top + ";left:" + left + ";"");
		 */
		return svg;
	}// end getRootNode

	// ----------------------------------------------------//

	// This is a utility method that is used to execute code
	// that is the same regardless of the graphic image
	// being produced.
	public static Document createDocument() {
		Document document = null;
		try {
			/*
			 * DocumentBuilderFactory factory = DocumentBuilderFactory
			 * .newInstance();
			 * 
			 * DocumentBuilder builder = factory.newDocumentBuilder(); document
			 * = builder.newDocument(); document.setXmlStandalone(false);
			 */

			/*
			 * <?xml version="1.0" encoding="UTF-8" standalone="no"?> <svg
			 * xmlns:xlink="http://www.w3.org/1999/xlink"
			 * xmlns="http://www.w3.org/2000/svg"
			 * contentScriptType="text/ecmascript" width="800.0"
			 * zoomAndPan="magnify" contentStyleType="text/css" height="600.0"
			 * preserveAspectRatio="xMidYMid meet" version="1.0">
			 */
			// We need a Document
			// String svgNS = "http://www.w3.org/2000/svg";
			DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
			dbfac.setNamespaceAware(true);
			dbfac.setValidating(false);

			//<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
			//DocumentType svgDocType = new DocumentType();
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
		
			document = docBuilder.newDocument();
			

			document.createAttributeNS(xlinkNS, "xlink");

			Element root = document.createElementNS(svgNS, "svg");
			document.appendChild(root);

			// root.createAttributeNS(xlinkNS, "xlink");
			// root.setAttributeNS(xmlNS, "xmlns", svgNS);
			// root.setPrefix("xlink");
			root.setAttributeNS(xmlNS, "xmlns:xlink", xlinkNS);
			root.setAttributeNS(null, "contentScriptType", "text/ecmascript");
			root.setAttributeNS(null, "zoomAndPan", "magnify");
			root.setAttributeNS(null, "contentStyleType", "text/css");
			root.setAttributeNS(null, "preserveAspectRatio", "xMidYMid meet");
			root.setAttributeNS(null, "version", "1.1");

			// DOMImplementation impl =
			// SVGDOMImplementation.getDOMImplementation();
			// document = impl.createDocument(svgNS, "svg", null);

		} catch (Exception e) {
			e.printStackTrace(System.err);
			System.exit(0);
		}// end catch
		return document;
	}// end getDocument

	/*
	 * <image";this << " x=\"0\"";this << " y=\"0\"";this << " height=\"" <<
	 * this->_gel_image_height << "\"";this << " width=\"" <<
	 * this->_gel_image_width << "\"";this <<
	 * " xlink:href=\"data:image/jpg;base64,"<< endl << this->_encoded_gel_image
	 * << endl <<"\"";this << "/>
	 */
	public static Element svgImage(Document document, Element parent,
			float xcoord, float ycoord, float width, float height, String href,
			String classname) throws Exception {

		// create the rectangle
		Element image = document.createElementNS(svgNS, "image");
		parent.appendChild(image);
		image.setAttributeNS(null, "x", "" + xcoord);
		image.setAttributeNS(null, "y", "" + ycoord);
		image.setAttributeNS(null, "width", "" + width);
		image.setAttributeNS(null, "height", "" + height);
		if (classname != null) {
			if (!classname.equals("")) {
				image.setAttributeNS(null, "class", classname);
			}
		}

		if (href == null) {
			throw new Exception(
					"svgImage(Document document, Element parent, float xcoord, float ycoord, float width, float height, URL href, String classname) href is NULL");
		}
		image.setAttributeNS(xlinkNS, "xlink:href", href);

		return image;
	}

	// ----------------------------------------------------//

	// This is a utility method that is used to execute code
	// that is the same regardless of the graphic image
	// being produced. This method transforms the DOM into
	// raw XML code and writes that code into the output.
	static void transformTheDom(Document document, String filename) {
		try {
			// Get a TransformerFactory object.
			TransformerFactory xformFactory = TransformerFactory.newInstance();

			// Get an XSL Transformer object.
			Transformer transformer = xformFactory.newTransformer();

			// Sets the standalone property in the first line of
			// the output file.
			transformer.setOutputProperty(OutputKeys.STANDALONE, "no");

			// Get a DOMSource object that represents the
			// Document object
			DOMSource source = new DOMSource(document);

			// Get a StreamResult object that points to the
			// screen. Then transform the DOM sending XML to
			// the screen.
			StreamResult scrResult = new StreamResult(System.out);
			transformer.transform(source, scrResult);

			// Get an output stream for the output file.
			PrintWriter outStream = new PrintWriter(filename);

			// Get a StreamResult object that points to the
			// output file. Then transform the DOM sending XML
			// code to the file
			StreamResult fileResult = new StreamResult(outStream);
			transformer.transform(source, fileResult);
		}// end try block

		catch (Exception e) {
			e.printStackTrace(System.err);
		}// end catch
	}// end transformTheDom

	// ----------------------------------------------------//

	public static void docToOutputStream(Document document, OutputStream out) {
		try {
			// Get a TransformerFactory object.
			TransformerFactory xformFactory = TransformerFactory.newInstance();

			// Get an XSL Transformer object.
			Transformer transformer = xformFactory.newTransformer();

			// Sets the standalone property in the first line of
			// the output file.
			//transformer.setOutputProperty(OutputKeys.STANDALONE, "no");
			transformer.setOutputProperty(OutputKeys.METHOD, "xml");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
			transformer.setOutputProperty(OutputKeys.VERSION, "1.0");
			//OutputKeys.OMIT_XML_DECLARATION

			//<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
			transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
			transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC, "-//W3C//DTD SVG 1.1//EN");
			// Get a DOMSource object that represents the
			// Document object

			DOMSource source = new DOMSource(document.getDocumentElement());

			// Get a StreamResult object that points to the
			// screen. Then transform the DOM sending XML to
			// the screen.
			StreamResult scrResult = new StreamResult(out);
			transformer.transform(source, scrResult);

		}// end try block

		catch (Exception e) {
			e.printStackTrace(System.err);
		}// end catch
	}// end transformTheDom

	public Element text(float xCoor, float yCoor, String content,
			String classname) throws Exception {
		if (currentNode == null) {
			throw new Exception("currentNode is null");
		}
		if (content == null) {
			throw new Exception("content is null");
		}
		Element text = svgText(this.svgdoc, currentNode, xCoor, yCoor,
				classname);
		svgTspan(this.svgdoc, text, xCoor, yCoor, content, "");
		return text;
	}

	public Element text(float xCoor, float yCoor, String content,
			String classname, int letter_space_in_px) {
		Element text = svgText(this.svgdoc, currentNode, xCoor, yCoor,
				classname);

		for (Character car : content.toCharArray()) {
			svgTspan(this.svgdoc, text, xCoor, yCoor, car.toString(), "");
			xCoor += letter_space_in_px;
		}
		return text;
	}

	public Element anchor(URL href) {
		return svgAnchor(this.svgdoc, currentNode, href);
	}

	public Element line(float xBeginCoor, float yBeginCoor, float xEndCoor,
			float yEndCoor, String classname) {
		return svgLine(this.svgdoc, currentNode, xBeginCoor, yBeginCoor,
				xEndCoor, yEndCoor, classname);
	}

	public Element image(float xcoord, float ycoord, float width, float height,
			URL href, String classname) throws Exception {
		return svgImage(this.svgdoc, currentNode, xcoord, ycoord, width,
				height, href.toString(), classname);
	}

	public Element embedJpgImage(float xcoord, float ycoord, float width,
			float height, File image_jpg_filename, String classname)
			throws Exception {

		// StringBuffer encoded_image_buffer = new StringBuffer(
		// "data:image/jpg;base64,");
		// encoded_image_buffer.append("\n");
		try {
			/*
			 * byte[] buffer_code = new byte[1024];
			 * 
			 * InputStream is = new BufferedInputStream(new FileInputStream(
			 * image_jpg_filename)); while ((is.read(buffer_code)) != -1) { //
			 * String s = new sun.misc.BASE64Encoder().encode(buffer_code);
			 * String s = Base64.encodeBytes(buffer_code);
			 * encoded_image_buffer.append(s); //
			 * encoded_image_buffer.append("\n"); }
			 */
			// encoded_image_buffer.append(Base64
			// .encodeFromFile(image_jpg_filename.toString()));
			BufferedImage img = ImageIO.read(image_jpg_filename);
			return this.embedJpgImage(xcoord, ycoord, width, height, img,
					classname);

		} catch (IOException e) {
			System.out.println(e.getMessage());
		}
		return null;
	}

	public Element embedJpgImage(float xcoord, float ycoord, float width,
			float height, BufferedImage img, String classname) throws Exception {
		StringBuffer encoded_image_buffer = new StringBuffer(
				"data:image/jpg;base64,");
		// encoded_image_buffer.append("\n");

		try {
			/*
			 * byte[] buffer_code = new byte[1024];
			 * 
			 * InputStream is = new BufferedInputStream(new FileInputStream(
			 * image_jpg_filename)); while ((is.read(buffer_code)) != -1) { //
			 * String s = new sun.misc.BASE64Encoder().encode(buffer_code);
			 * String s = Base64.encodeBytes(buffer_code);
			 * encoded_image_buffer.append(s); //
			 * encoded_image_buffer.append("\n"); }
			 */

			ByteArrayOutputStream out = new ByteArrayOutputStream();
			ImageIO.write(img, "jpg", out);
			// byte[] bytes = out.toByteArray();

			encoded_image_buffer.append(Base64.encodeBytes(out.toByteArray()));
			// .encodeFromFile(img.image_jpg_filename.toString()));

		} catch (IOException e) {
			System.out.println(e.getMessage());
		}

		if (encoded_image_buffer.length() > 0) {

		} else {
			throw new Exception("encoded_image_buffer empty");
		}

		return svgImage(this.svgdoc, currentNode, xcoord, ycoord, width,
				height, encoded_image_buffer.toString(), classname);
	}

	public SvgGroup group(String groupid, String classname) {
		SvgGroup new_group = new SvgGroup(this.svgdoc, currentNode, groupid,
				classname);
		this.groups.put(groupid, new_group);
		return new_group;
	}

	public SvgGroup goToGroup(String groupid) throws Exception {
		SvgGroup gotogroup = null;
		if (this.groups.containsKey(groupid)) {
			gotogroup = this.groups.get(groupid);
			if (gotogroup == null) {
				throw new Exception("Error in SvgGraphics : the group id "
						+ groupid + " was not found");
			}
			this.currentNode = gotogroup.getElement();
		} else {
			throw new Exception("Error in SvgGraphics : the group id "
					+ groupid + " was not found");
		}
		return gotogroup;
	}

	public void toOutputStream(OutputStream out) {
		docToOutputStream(this.svgdoc, out);
	}

	public void setCssClass(String classname, String classdefinition) {
		this.cssClasses.put(classname, classdefinition);

		updateCss();
	}

	private void updateCss() {
		// Add a CDATA section to the root element
		String csssyntax = "\n";

		Set<Entry<String, String>> lesEntrees = this.cssClasses.entrySet();
		Iterator<Entry<String, String>> it = lesEntrees.iterator();
		while (it.hasNext()) {

			Map.Entry<String, String> e = (Map.Entry<String, String>) it.next();

			csssyntax += "\n" + e.getKey() + " {" + e.getValue() + "}";

			// la clé de l'élement courant est e . getKey ( ) ;

			// la valeur de l'élement courant est e . getValue ( )

		}
		csssyntax += "\n";
		// System.out.print("cestca: "+csssyntax);

		// cdataStyle.deleteData(0, cdataStyle.getLength());
		cdataStyle.setData(csssyntax);

	}

	public void toFile(File filename) {
		// Stream to write file
		FileOutputStream fout;

		try {
			// Open an output stream
			fout = new FileOutputStream(filename.getAbsolutePath());

			// Print a line of text
			this.toOutputStream(fout);

			// Close our output stream
			fout.close();
		}
		// Catches any error conditions
		catch (IOException e) {
			System.err.println("Unable to write to file");
			System.exit(-1);
		}

	}

	public Document getDocument() {
		return this.svgdoc;
	}

	public float getWidth() {
		return this.width;
	}

	public float getHeight() {
		return this.height;
	}

	public Element getCurrentNode() {
		return this.currentNode;
	}

	public Element scientificNumber(int xCoor, float yCoor, String content,
			String classname) throws Exception {
		return this.text(xCoor, yCoor, content, classname);
		/*
		 * Element text = svgText(this.svgdoc, currentNode, xCoor, yCoor,
		 * classname); String[] arr_number = content.split("E");
		 * svgTspan(this.svgdoc, text, null, null, arr_number[0]+"e", "");
		 * Element tspan = svgTspan(this.svgdoc, text, null, null,
		 * arr_number[1], "exponential"); tspan.setAttributeNS(null, "dx", "0");
		 * tspan.setAttributeNS(null, "style", "baseline-shift:super");
		 */
		// return text;
	}

	// ----------------------------------------------------//

	public Element symbol(String id) {
		/*
		 * <symbol id="MySymbol" viewBox="0 0 20 20"> <desc>MySymbol - four
		 * rectangles in a grid</desc> <rect x="1" y="1" width="8" height="8"/>
		 * <rect x="11" y="1" width="8" height="8"/> <rect x="1" y="11"
		 * width="8" height="8"/> <rect x="11" y="11" width="8" height="8"/>
		 * </symbol>
		 */
		Element symbol = this.svgdoc.createElementNS(svgNS, "symbol");
		this.defs.appendChild(symbol);
		symbol.setAttributeNS(null, "id", id);

		hash_defs.put(id, symbol);
		return symbol;
	}

	public Element symbol(String id, float xcoord, float ycoord, float width,
			float height, String preserve_ratio) {
		/*
		 * <symbol id="MySymbol" viewBox="0 0 20 20"> <desc>MySymbol - four
		 * rectangles in a grid</desc> <rect x="1" y="1" width="8" height="8"/>
		 * <rect x="11" y="1" width="8" height="8"/> <rect x="1" y="11"
		 * width="8" height="8"/> <rect x="11" y="11" width="8" height="8"/>
		 * </symbol>
		 */
		Element symbol = this.symbol(id);
		symbol.setAttributeNS(null, "viewBox", "" + xcoord + " " + ycoord + " "
				+ width + " " + height);
		if (preserve_ratio != null) {
			symbol.setAttributeNS(null, "preserveAspectRatio", ""
					+ preserve_ratio);

		}
		return symbol;
	}

	public Element getSymbol(String key) throws Exception {
		Element symbol = this.hash_defs.get(key);
		return symbol;
	}

	public Element use(float x, float y, Element symbol) {
		/*
		 * <use x="45" y="10" width="10" height="10" xlink:href="#MySymbol" />
		 */

		Element use = this.svgdoc.createElementNS(svgNS, "use");
		this.currentNode.appendChild(use);
		use.setAttributeNS(null, "x", "" + x);
		use.setAttributeNS(null, "y", "" + y);
		use.setAttributeNS(xlinkNS, "xlink:href",
				"#" + symbol.getAttributeNS(null, "id"));
		return use;
	}

}// end class SvgGraphics

