package fr.inra.moulon.xhtmlutils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.util.HashMap;

import org.w3c.dom.*;

import fr.inra.moulon.svgutils.SvgCommon;
import fr.inra.moulon.xhtmlutils.xhtmltable.XhtmlTable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

//This is a small proof-of-concept graphics class that
// provides method calls for the creation of the following
// DOM tree nodes:
//  A simple element with no attributes.
//  A linear gradient element.
//  An ellipse
//  A circle

//Each method receives a reference to the overall document
// along with a reference to the parent for the new node.
//When the method returns, the new node has been appended
// to the parent node.

public class XhtmlDocument implements SvgCommon {

	public static String xhtmlNS = "http://www.w3.org/1999/xhtml";
	public static String xlinkNS = "http://www.w3.org/1999/xlink";
	public static String xmlNS = "http://www.w3.org/2000/xmlns/";

	private Document xhtmldoc;
	private Element xhtmlroot;
	private Element currentNode;
	private Element style;
	private CDATASection cdataStyle;

	HashMap<String, HashMap<String, String>> cssClasses = new HashMap<String, HashMap<String, String>>(
			0);

	public XhtmlDocument() {

		// String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
		// DOMImplementation impl = SVGDOMImplementation.getDOMImplementation();
		// svgdoc = (Document) impl.createDocument(svgNS, "svg", null);

		xhtmldoc = createDocument();
		xhtmlroot = xhtmldoc.getDocumentElement();
		Element head = xhtmldoc.createElementNS(xhtmlNS, "head");
		xhtmlroot.appendChild(head);

		/*
		 * <head> <meta http-equiv="Content-Type"
		 * content="text/html; charset=utf-8"/> <title>Bioinformatique </title>
		 * <link rel="stylesheet" href="../css/3ps.css" type="text/CSS"/> <link
		 * rel="stylesheet" href="../css/pappso_position.css" type="text/CSS"/>
		 * <link rel="shortcut icon" href="../images/logo/pappso_icon.png"
		 * type="images/x-icon"/> </head>
		 */
		Element body = xhtmldoc.createElementNS(xhtmlNS, "body");
		xhtmlroot.appendChild(body);

		currentNode = body;

		// <style type="text/css"> ... </style>

		style = xhtmlStyle(xhtmldoc, head);
		// defs = svgDefs(svgdoc, svgroot);
		cdataStyle = this.xhtmldoc.createCDATASection(" ");
		style.appendChild(cdataStyle);

		/*
		 * this.setCssClass( "svg",
		 * "fill:none;fill-rule:evenodd;stroke:#000000;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1"
		 * );
		 */

	}

	public static Element xhtmlAnchor(Document document, Element parent,
			URL href) {
		// <a xlink:href="/" xlink:title="Go Home">
		Element anchor = document.createElementNS(xhtmlNS, "a");
		parent.appendChild(anchor);
		anchor.setAttributeNS(xlinkNS, "xlink:href", href.toString());
		return anchor;
	}

	// ----------------------------------------------------//

	// This is a utility method that is used to execute code
	// that is the same regardless of the graphic image
	// being produced.
	public static Document createDocument() {
		Document document = null;
		try {
			/*
			 * DocumentBuilderFactory factory = DocumentBuilderFactory
			 * .newInstance();
			 * 
			 * DocumentBuilder builder = factory.newDocumentBuilder(); document
			 * = builder.newDocument(); document.setXmlStandalone(false);
			 */

			/*
			 * <?xml version="1.0" encoding="UTF-8" standalone="no"?> <svg
			 * xmlns:xlink="http://www.w3.org/1999/xlink"
			 * xmlns="http://www.w3.org/2000/svg"
			 * contentScriptType="text/ecmascript" width="800.0"
			 * zoomAndPan="magnify" contentStyleType="text/css" height="600.0"
			 * preserveAspectRatio="xMidYMid meet" version="1.0">
			 */
			// We need a Document
			// String svgNS = "http://www.w3.org/2000/svg";
			DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
			dbfac.setNamespaceAware(true);
			dbfac.setValidating(false);

			// <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"
			// "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
			// DocumentType svgDocType = new DocumentType();
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();

			document = docBuilder.newDocument();

			document.createAttributeNS(xlinkNS, "xlink");

			Element root = document.createElementNS(xhtmlNS, "xhtml");
			document.appendChild(root);

			// root.createAttributeNS(xlinkNS, "xlink");
			// root.setAttributeNS(xmlNS, "xmlns", svgNS);
			// root.setPrefix("xlink");
			root.setAttributeNS(xmlNS, "xmlns:xlink", xlinkNS);
			// root.setAttributeNS(null, "contentScriptType",
			// "text/ecmascript");
			// root.setAttributeNS(null, "zoomAndPan", "magnify");
			// root.setAttributeNS(null, "contentStyleType", "text/css");
			// root.setAttributeNS(null, "preserveAspectRatio",
			// "xMidYMid meet");
			root.setAttributeNS(null, "version", "1.1");

			// <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">

			// DOMImplementation impl =
			// SVGDOMImplementation.getDOMImplementation();
			// document = impl.createDocument(svgNS, "svg", null);

		} catch (Exception e) {
			e.printStackTrace(System.err);
			System.exit(0);
		}// end catch
		return document;
	}// end getDocument

	/*
	 * <image";this << " x=\"0\"";this << " y=\"0\"";this << " height=\"" <<
	 * this->_gel_image_height << "\"";this << " width=\"" <<
	 * this->_gel_image_width << "\"";this <<
	 * " xlink:href=\"data:image/jpg;base64,"<< endl << this->_encoded_gel_image
	 * << endl <<"\"";this << "/>
	 */
	public static Element xhtmlImage(Document document, Element parent,
			float xcoord, float ycoord, float width, float height, String href,
			String classname) throws Exception {

		// create the rectangle
		Element image = document.createElementNS(xhtmlNS, "image");
		parent.appendChild(image);
		image.setAttributeNS(null, "x", "" + xcoord);
		image.setAttributeNS(null, "y", "" + ycoord);
		image.setAttributeNS(null, "width", "" + width);
		image.setAttributeNS(null, "height", "" + height);
		if (classname != null) {
			if (!classname.equals("")) {
				image.setAttributeNS(null, "class", classname);
			}
		}

		if (href == null) {
			throw new Exception(
					"xhtmlImage(Document document, Element parent, float xcoord, float ycoord, float width, float height, URL href, String classname) href is NULL");
		}
		image.setAttributeNS(xlinkNS, "xlink:href", href);

		return image;
	}

	// ----------------------------------------------------//

	// This is a utility method that is used to execute code
	// that is the same regardless of the graphic image
	// being produced. This method transforms the DOM into
	// raw XML code and writes that code into the output.
	static void transformTheDom(Document document, String filename) {
		try {
			// Get a TransformerFactory object.
			TransformerFactory xformFactory = TransformerFactory.newInstance();

			// Get an XSL Transformer object.
			Transformer transformer = xformFactory.newTransformer();

			// Sets the standalone property in the first line of
			// the output file.
			transformer.setOutputProperty(OutputKeys.STANDALONE, "no");

			// Get a DOMSource object that represents the
			// Document object
			DOMSource source = new DOMSource(document);

			// Get a StreamResult object that points to the
			// screen. Then transform the DOM sending XML to
			// the screen.
			StreamResult scrResult = new StreamResult(System.out);
			transformer.transform(source, scrResult);

			// Get an output stream for the output file.
			PrintWriter outStream = new PrintWriter(filename);

			// Get a StreamResult object that points to the
			// output file. Then transform the DOM sending XML
			// code to the file
			StreamResult fileResult = new StreamResult(outStream);
			transformer.transform(source, fileResult);
		}// end try block

		catch (Exception e) {
			e.printStackTrace(System.err);
		}// end catch
	}// end transformTheDom

	// ----------------------------------------------------//

	public static void docToOutputStream(Document document, OutputStream out) {
		try {
			// Get a TransformerFactory object.
			TransformerFactory xformFactory = TransformerFactory.newInstance();

			// Get an XSL Transformer object.
			Transformer transformer = xformFactory.newTransformer();

			// Sets the standalone property in the first line of
			// the output file.
			// transformer.setOutputProperty(OutputKeys.STANDALONE, "no");
			transformer.setOutputProperty(OutputKeys.METHOD, "xml");
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");
			transformer
					.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
			transformer.setOutputProperty(OutputKeys.VERSION, "1.0");
			// OutputKeys.OMIT_XML_DECLARATION

			// <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
			// "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
			// <!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"
			// "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">
			transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM,
					"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd");
			transformer.setOutputProperty(OutputKeys.DOCTYPE_PUBLIC,
					"-//W3C//DTD XHTML 1.0 Transitional//EN");
			// Get a DOMSource object that represents the
			// Document object

			DOMSource source = new DOMSource(document.getDocumentElement());

			// Get a StreamResult object that points to the
			// screen. Then transform the DOM sending XML to
			// the screen.
			StreamResult scrResult = new StreamResult(out);
			transformer.transform(source, scrResult);

		}// end try block

		catch (Exception e) {
			e.printStackTrace(System.err);
		}// end catch
	}// end transformTheDom

	public Element anchor(URL href) {
		return xhtmlAnchor(this.xhtmldoc, currentNode, href);
	}

	public Element image(float xcoord, float ycoord, float width, float height,
			URL href, String classname) throws Exception {
		return xhtmlImage(this.xhtmldoc, currentNode, xcoord, ycoord, width,
				height, href.toString(), classname);
	}

	public void toOutputStream(OutputStream out) {
		writeCss();

		docToOutputStream(this.xhtmldoc, out);
	}

	public void setCssClass(String cssKey, String cssProperty, String cssValue) {
		HashMap<String, String> the_css;
		if (cssClasses.containsKey(cssKey)) {
			the_css = cssClasses.get(cssKey);
		}
		else {
			the_css = new HashMap<String, String>(0);
			cssClasses.put(cssKey, the_css);
		}
		the_css.put(cssProperty, cssValue);
		
	}

	public void setCssClass(String classname, String classdefinition) {

		String[] props = classdefinition.split(";");
		HashMap<String, String> the_css = new HashMap<String, String>(0);
		cssClasses.put(classname, the_css);
		

		for (String css : props) {
			String[] property = css.split(":", 2);
			the_css.put(property[0], property[1]);
		}

		// this.cssClasses.put(classname, classdefinition);
	}

	private void writeCss() {
		// Add a CDATA section to the root element
		StringBuilder csssyntax = new StringBuilder("\n");

		for (String classKey : this.cssClasses.keySet()) {

			csssyntax.append("\n" + classKey + " {");

			HashMap<String, String> cssClass = cssClasses.get(classKey);
			for (String propertyKey : cssClass.keySet()) {
				csssyntax.append(propertyKey + ":" + cssClass.get(propertyKey)
						+ ";");
			}

			csssyntax.append("}");

			// la clé de l'élement courant est e . getKey ( ) ;

			// la valeur de l'élement courant est e . getValue ( )

		}
		csssyntax.append("\n");
		// System.out.print("cestca: "+csssyntax);

		// cdataStyle.deleteData(0, cdataStyle.getLength());
		cdataStyle.setData(csssyntax.toString());

	}

	public void toFile(File filename) {
		// Stream to write file
		FileOutputStream fout;

		try {
			// Open an output stream
			fout = new FileOutputStream(filename.getAbsolutePath());

			// Print a line of text
			this.toOutputStream(fout);

			// Close our output stream
			fout.close();
		}
		// Catches any error conditions
		catch (IOException e) {
			System.err.println("Unable to write to file");
			System.exit(-1);
		}

	}

	public Document getDocument() {
		return this.xhtmldoc;
	}

	public Element getCurrentNode() {
		return this.currentNode;
	}

	public static Element xhtmlStyle(Document document, Element parent) {

		// create the rectangle
		Element rect = document.createElementNS(xhtmlNS, "style");
		parent.appendChild(rect);
		rect.setAttributeNS(null, "type", "text/css");
		return rect;
	}

	public XhtmlTable newTable() {

		XhtmlTable xhtmlTable = new XhtmlTable(this);

		this.getCurrentNode().appendChild(xhtmlTable.getElement());

		return xhtmlTable;

	}

	public void includeSvgDocument(SvgCommon mon_svg) throws DOMException,
			Exception {
		// Document svgDoc = spectrum.getDocument();
		// Node svg = svgDoc.cloneNode(true);

		Node svgInXhtml = getDocument().importNode(
				mon_svg.getDocument().getDocumentElement(), true);

		// xhtmlDoc.getCurrentNode().appendChild(svgInXhtml);
		Element root2 = this.getCurrentNode();
		root2.appendChild(svgInXhtml);

	}

}// end class SvgGraphics

